//
//  FolderItem.c
//  file-folder
//
//  Created by kunii on 2014/01/18.
//  Copyright (c) 2014年 國居貴浩. All rights reserved.
//

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include "FolderItem.h"

static Item* folder_super = NULL;

//  Item構造体を初期化する。
//      nameに9バイト以上の文字列をわたしてはいけない
static Item* initFolderItem(Item* item, const char* name)
{
    folder_super->init(item, name);
    FolderItem* folderItem = (FolderItem*)item;
    folderItem->subItemCount = 0;         //  自分が内包するフォルダやファイルの数を0にする
    folderItem->subItems = NULL;          //  自分が内包するフォルダやファイルの配列は初期状態では持たないのでNULLにする
    return item;
}

//  Item構造体を破棄する。
static void deallocFolderItem(Item* item)
{
    FolderItem* folderItem = (FolderItem*)item;
    if (folderItem->subItems) {
        //  自分が内包するItem構造体群を一緒に破棄する
        for (int i = 0; i < folderItem->subItemCount; i++) {
            folderItem->subItems[i]->dealloc(folderItem->subItems[i]); //  deallocItemの再帰処理
        }
        //  subItemsに確保したItem構造体ポインタ配列用メモリ領域を破棄する
        free(folderItem->subItems);
    }
    folder_super->dealloc(item);
}

//  itemで指定されるItem構造体の名前をコンソールに表示する。
//  もし内包するフォルダやファイル群があれば、itemの名前より半角スペース1つずらして名前を表示する。
//      例）
//      itemの名前
//       itemが内包するフォルダやファイルの名前 1
//       itemが内包するフォルダやファイルの名前 2
//
static void printFolderItem(Item* item, int indent)
{
    //  indentで指定される数だけ半角スペースを表示してから名前を表示する。
    for (int i = 0; i < indent; i++) printf(" ");
    printf("%s\n", item->name);
    
    FolderItem* folderItem = (FolderItem*)item;
   //  もし内包するフォルダやファイル群があれば、1つ半角スペースをあけて名前を表示する。
    indent++;
    for (int i = 0; i < folderItem->subItemCount; i++) {
        Item* subitem = folderItem->subItems[i];
        subitem->print(subitem, indent);   //  subitemはsubitemのprintを使う
    }
}

//  itemで指定されるItem構造体に、subItemで指定されるItem構造体を内包させる。
static void addFolderItem(FolderItem* folderItem, Item* subItem)
{
    //  新しいItem構造体ポインタ配列用メモリ領域をmallocで確保する。
    Item** subItems = malloc(sizeof(Item*) * (folderItem->subItemCount + 1));
    if (folderItem->subItems) {
        //  もともとのItem構造体ポインタ配列用メモリ領域は、各ポインタの値を新しい方にコピーしてから破棄する。
        for (int i = 0; i < folderItem->subItemCount; i++) {
            subItems[i] = folderItem->subItems[i];
        }
        free(folderItem->subItems);   //  破棄
    }
    //  subItemCountを1つ増やしつつ、配列最後尾にsubItemを設定する。
    subItems[folderItem->subItemCount++] = subItem;
    folderItem->subItems = subItems;  //  設定
}

Item* FolderItemAlloc()
{
    if (folder_super == NULL)
        folder_super = ItemAlloc();
    Item* item = malloc(sizeof(FolderItem));
    item->init = initFolderItem;
    item->print = printFolderItem;
    item->dealloc = deallocFolderItem;
    FolderItem* folderItem = (FolderItem*)item;
    folderItem->add = addFolderItem;
    return item;
}

